#' Prediction interval for future trial
#'
#' Given a model fit, compute prediction and standard error of the prediction
#' for a future trial. Both the standard error for the mean, and the standard
#' error for the prediction are computed.
#'
#' @param fit4 Model fit object as returned by \link{fit_model4}
#' @param v Value at which prediction is desired. This is a log-odds ratio
#' @param sig0 Variance of v
#'
#' @return A list with elements:
#' \describe{
#'  \item{delta}{The predicted value}
#'  \item{se.delta}{The standard error of the prediction (confidence interval)}
#'  \item{se.pred}{The standard error of a new observation (prediction interval)}
#'  \item{a3}{The third component in the sum for the variance, for testing}
#'  }
#'
#' @export

predict_klmfit <- function(fit4, v, sig0){

  delta <- fit4$par["muy"] + fit4$par["beta"] * v * (fit4$par["vmx"] / (fit4$par["vmx"] + sig0)) +
    fit4$par["beta"] * fit4$par["mux"] * (1 - fit4$par["vmx"] / (fit4$par["vmx"] + sig0))

  ## delta method

  mu.deriv <- deriv(expression(muy + beta * v * (vmx/(vmx + sig0)) +
                                 beta * mux * (1 - vmx/(vmx + sig0))),
                    namevec = c("mux", "muy", "beta", "vmx", "vmg"),
                    function.arg = function(mux, muy, beta, vmx, vmg, v, sig0){} )

  delta <- do.call(mu.deriv, as.list(c(fit4$par, v, sig0)))
  mu.grad <- attr(delta, "gradient")
  var.delta <- c(mu.grad %*% solve(fit4$hessian) %*% t(mu.grad))

  a3 <- fit4$par["beta"]^2 * fit4$par["vmx"] *
    (1 - sqrt(fit4$par["vmx"])/sqrt(fit4$par["vmx"] + sig0))

  tilde.v <- var.delta + fit4$par["vmg"] + a3


  list(delta = delta, se.delta = sqrt(var.delta), se.pred = sqrt(tilde.v), a3 = a3)

}

#' Fill prediction table
#'
#' Given pCR proportions in two groups, sample size, and model fit, calculates
#' log odds ratio and forms predictions for treatment effect on the definitive
#' outcome
#'
#' @param p1 pCR probability in the treatment group
#' @param p0 pCR probability in the control group
#' @param n Sample size
#' @param fit Model fit as returned by \link{fit_model4}
#'
#' @return A list with elements:
#' \describe{
#'  \item{a1}{The prediction and interval, in string form}
#'  \item{a2}{Vector of variance components, for testing}
#'}
#'
#' @export

fill_table <- function(p1, p0, n, fit){

  a <- n/2 * p0
  b <- n/2 * p1
  c <- n/2 - a
  d <- n/2 - b

  logOR <- -log(a * d / (b * c))
  sig0 <- sum(1/c(a, b, c, d))
  psm <- predict_klmfit(fit, logOR, sig0)
  tf <- qt(.975, df = fit$n - 3)

  # assume 1/3 the subjects have events and the are split evenly

  n1 <- n * .15

  se.hr <- sqrt(1/n1 + 1/n1)

  ci <- as.list(exp(psm$delta + c(0, -tf, tf) * psm$se.pred))
  ci2 <- as.list(exp(psm$delta + c(0, -tf, tf) * sqrt(psm$se.pred^2 + se.hr^2)))
  a1 <- do.call(sprintf, c("%.2f (%.2f, %.2f)",ci))
  a3 <- do.call(sprintf, c("%.2f (%.2f, %.2f)",ci2))
  a2 <- c(psm$se.delta^2, fit$par["vmg"], psm$a3)
  list(a1 = a1, a2 = a2, a3 = a3)

}
